/**
* @license Apache-2.0
*
* Copyright (c) 2018 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

// MAIN //

var vtree = {
	'tagName': 'g',
	'properties': {
		'property': 'rug',
		'className': 'rug',
		'namespace': void 0
	},
	'children': [
		{
			'tagName': 'line',
			'properties': {
				'property': 'rug.tick',
				'className': 'tick',
				'attributes': {
					'fill': 'none',
					'opacity': 0.9,
					'stroke': '#aaa',
					'stroke-width': 1,
					'data-label': '',
					'x1': 0,
					'x2': -6,
					'y1': 0.1,
					'y2': 0.1
				},
				'namespace': void 0
			},
			'children': [],
			'namespace': 'http://www.w3.org/2000/svg',
			'count': 0,
			'hasWidgets': false,
			'hasThunks': false,
			'descendantHooks': false,
			'hooks': void 0,
			'key': void 0
		},
		{
			'tagName': 'line',
			'properties': {
				'property': 'rug.tick',
				'className': 'tick',
				'attributes': {
					'fill': 'none',
					'opacity': 0.9,
					'stroke': '#aaa',
					'stroke-width': 1,
					'data-label': '',
					'x1': 0,
					'x2': -6,
					'y1': 0.5,
					'y2': 0.5
				},
				'namespace': void 0
			},
			'children': [],
			'namespace': 'http://www.w3.org/2000/svg',
			'count': 0,
			'hasWidgets': false,
			'hasThunks': false,
			'descendantHooks': false,
			'hooks': void 0,
			'key': void 0
		},
		{
			'tagName': 'line',
			'properties': {
				'property': 'rug.tick',
				'className': 'tick',
				'attributes': {
					'fill': 'none',
					'opacity': 0.9,
					'stroke': '#aaa',
					'stroke-width': 1,
					'data-label': '',
					'x1': 0,
					'x2': -6,
					'y1': 0.9,
					'y2': 0.9
				},
				'namespace': void 0
			},
			'children': [],
			'namespace': 'http://www.w3.org/2000/svg',
			'count': 0,
			'hasWidgets': false,
			'hasThunks': false,
			'descendantHooks': false,
			'hooks': void 0,
			'key': void 0
		}
	],
	'namespace': 'http://www.w3.org/2000/svg',
	'count': 3,
	'hasWidgets': false,
	'hasThunks': false,
	'descendantHooks': false,
	'hooks': void 0,
	'key': void 0
};


// EXPORTS //

module.exports = vtree;
