/**
* @license Apache-2.0
*
* Copyright (c) 2018 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

// MODULES //

var FLOAT64_MAX = require( '@stdlib/constants/float64/max' );
var isDefined = require( './accessors/is_defined.js' );


// MAIN //

/**
* Returns default options.
*
* @private
* @returns {Object} default options
*/
function defaults() {
	var out = {};

	// Boolean indicating whether to re-render on a `change` event:
	out.autoRender = false;

	// Data buffer size:
	out.bufferSize = FLOAT64_MAX;

	// Sparkline data:
	out.data = [];

	// Sparkline description:
	out.description = '';

	// Accessor indicating whether a datum is defined:
	out.isDefined = isDefined;

	// Data label:
	out.label = '';

	return out;
}


// EXPORTS //

module.exports = defaults;
